﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.HelperClasses;
using SmarterNotify.DataHandlers;
using SmarterNotify.Connector;

namespace SmarterNotify.Data
{
    public class ConnectionProfile
    {
        public enum CustomVariableTypes
        {
            PluginType
        }
        public enum ConnectionProfileType
        {
            SmarterMail,
            SmarterTrack,
            SmarterStats,
            ExternalProvider,
            PluginProvider
        }
        public enum ConnectionStatusType
        {
            Started,
            Stopped,
            Paused
        }

        public string ProfileGUID { get; set; }
        public string ProfileName { get; set; }
        private List<Reminder> LastRemindersConnected { get; set; }
        public ConnectionProfileType ConnectionType { get; set; }
        public string ConnectionURL { get; set; }
        public string AuthUsername { get; set; }
        public string AuthPassword { get; set; }
        public string DataUsername { get; set; }
        public bool StartAppConnected { get; set; }
        public ConnectionStatusType ConnectionStatus { get; set; }
        public Dictionary<CustomVariableTypes, string> CustomVars { get; set; }
        public bool IsPendingConnection { get; set; }
        private bool IsPlugin { get { return ConnectionType == ConnectionProfileType.PluginProvider; } }

        public ConnectionProfile()
        {
            ProfileGUID = Guid.NewGuid().ToString("N");
            ProfileName = "";
            LastRemindersConnected = new List<Reminder>();
            ConnectionType = ConnectionProfileType.SmarterTrack;
            ConnectionURL = "";
            AuthUsername = "";
            AuthPassword = "";
            DataUsername = "";
            StartAppConnected = true;
            ConnectionStatus = ConnectionStatusType.Stopped;
            CustomVars = new Dictionary<CustomVariableTypes, string>();
            IsPendingConnection = false;
        }
        public void StartConnections(bool doNotify)
        {
            try
            {
                if (IsPlugin && PluginProvider == null)
                    return;

                TrackingHelper.RecordType recordType = TrackingHelper.RecordType.Log;
                if (ConnectionType == ConnectionProfileType.PluginProvider)
                    ConnectionStatus = PluginProvider.IsPluginAvailable() ? ConnectionStatusType.Started : ConnectionStatusType.Paused;
                else
                    ConnectionStatus = STWebServices.IsServiceAvailable(this) ? ConnectionStatusType.Started : ConnectionStatusType.Paused;
                
                string logMsg = "The " + ConnectionType.ToString() + " connection profile \"" + 
                    ProfileName + "\" is now connected and able to receive notifications";
                if (ConnectionStatus == ConnectionStatusType.Started)
                {
                    if (ConnectionType == ConnectionProfileType.PluginProvider)
                        PluginProvider.StartConnection();

                    if (doNotify)
                        recordType = TrackingHelper.RecordType.Both;
                    TrackingHelper.TrackConnection(recordType, "Connection Started", logMsg, logMsg);
                }
                IsPendingConnection = false;
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while running ConnectProfile() for profile \"" + ProfileName + "\"";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public bool IsServiceAvailable()
        {
            bool retVal = false;
            try
            {
                if (IsPlugin)
                    retVal = PluginProvider.IsPluginAvailable();
                else
                    retVal = STWebServices.IsServiceAvailable(this);
            }
            catch (Exception ex) 
            { 
                string logMsg = "Error occurred while running IsServiceAvailable() for profile " + ProfileName;
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
            return retVal;
        }
        public void StopConnections()
        {
            try
            {
                if (!IsPlugin)
                    LastRemindersConnected = GetAllReminders(true);

                ConnectionStatus = ConnectionStatusType.Stopped;
                if (ConnectionType == ConnectionProfileType.PluginProvider)
                    PluginProvider.StopConnection();

                string logMsg = "The " + ConnectionType.ToString() + " connection profile \"" + ProfileName + "\" is no longer receiving notifications";
                TrackingHelper.TrackConnection(TrackingHelper.RecordType.Log, "Connection Stopped", logMsg, logMsg);
                IsPendingConnection = false;
            }
            catch { }
        }
        public void PauseConnections()
        {
            try
            {
                ConnectionStatus = ConnectionStatusType.Paused;
                string logMsg = "The " + ConnectionType.ToString() + " connection profile \"" + ProfileName + "\" is not available at this time";
                TrackingHelper.TrackConnection(TrackingHelper.RecordType.Both, "Connection Unavailable", logMsg, logMsg);
                IsPendingConnection = false;
            }
            catch { }
        }
        public List<Reminder> GetAllReminders(bool includeSnoozedItems)
        {
            List<Reminder> retVal = new List<Reminder>();
            try
            {
                if (IsPlugin)
                    retVal = InternalReminderHandler.GetAllPluginReminders(includeSnoozedItems);
                else if (ConnectionStatus == ConnectionStatusType.Started)
                {
                    retVal = STWebServices.GetAllReminders(this, includeSnoozedItems);
                    LastRemindersConnected = retVal;
                }
                else
                    retVal = LastRemindersConnected;
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while running GetAllReminders() for profile \"" + ProfileName + "\"";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
            return retVal;
        }
        public Reminder GetReminder(string reminderID)
        {
            Reminder retVal = new Reminder();
            try
            {
                if (IsPlugin)
                    retVal = InternalReminderHandler.GetPluginReminder(reminderID);
                else if (ConnectionStatus == ConnectionStatusType.Started)
                {
                    foreach (Reminder reminder in LastRemindersConnected)
                        if (reminder.ReminderID == reminderID)
                            return reminder;

                    retVal = STWebServices.GetReminder(this, reminderID);
                }
                else
                {
                    // find this reminder in the list of last reminders
                    foreach (Reminder reminder in LastRemindersConnected)
                        if (reminder.ReminderID == reminderID)
                        {
                            retVal = reminder;
                            break;
                        }
                }
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while running GetReminder(int) for profile \"" + ProfileName + "\"";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
            return retVal;
        }

        #region Unused
        //public int GetReminderCount()
        //{
        //    int retVal = 0;
        //    if (ConnectionStatus == ConnectionStatusType.Started)
        //    {
        //        try
        //        {
        //            retVal = STWebServices.GetReminderCount(this);
        //            //switch (ConnectionType)
        //            //{
        //            //    case ConnectionProfileType.SmarterMail: break;
        //            //    case ConnectionProfileType.SmarterStats: break;
        //            //    case ConnectionProfileType.SmarterTrack: retVal = STWebServices.GetReminderCount(this); break;
        //            //}
        //        }
        //        catch (Exception ex) 
        //        {
        //            string logMsg = "Error occurred while running GetReminderCount() for profile \"" + ProfileName + "\"";
        //            TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
        //        }
        //    }
        //    else
        //        retVal = LastRemindersConnected.Count;
        //    return retVal;
        //}
        //public int GetNewReminderCount(DateTime lastCheckDateUTC)
        //{
        //    int retVal = 0;
        //    if (ConnectionStatus == ConnectionStatusType.Started)
        //    {
        //        try
        //        {
        //            retVal = STWebServices.GetNewReminderCount(this, lastCheckDateUTC);
        //            //switch (ConnectionType)
        //            //{
        //            //    case ConnectionProfileType.SmarterMail: break;
        //            //    case ConnectionProfileType.SmarterStats: break;
        //            //    case ConnectionProfileType.SmarterTrack: retVal = STWebServices.GetNewReminderCount(this, lastCheckDateUTC); break;
        //            //}
        //        }
        //        catch (Exception ex) 
        //        {
        //            string logMsg = "Error occurred while running GetNewReminderCount(DateTime) for profile \"" + ProfileName + "\"";
        //            TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
        //        }
        //    }
        //    return retVal;
        //}
        #endregion

        public void DismissAllReminders()
        {
            try
            {
                if (IsPlugin)
                    InternalReminderHandler.DismissAllPluginReminders(true);
                else if (ConnectionStatus == ConnectionStatusType.Started)
                    STWebServices.DismissAllReminders(this);
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while running DismissAllReminders() for profile \"" + ProfileName + "\"";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public void DissmissReminder(string reminderID)
        {
            try
            {
                if (IsPlugin)
                    InternalReminderHandler.DismissReminder(reminderID, true);
                else if (ConnectionStatus == ConnectionStatusType.Started)
                    STWebServices.DissmissReminder(this, reminderID);
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while running DissmissReminder(int) for profile \"" + ProfileName + "\"";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public void SnoozeReminder(string reminderID, int snoozeSeconds)
        {
            try
            {
                if (IsPlugin)
                    InternalReminderHandler.SnoozeReminder(reminderID, snoozeSeconds, true);
                else if (ConnectionStatus == ConnectionStatusType.Started)
                    STWebServices.SnoozeReminder(this, reminderID, snoozeSeconds);
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while running SnoozeReminder(int, int) for profile \"" + ProfileName + "\"";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }

        #region Helper Methods

        public INotifyPluginProvider PluginProvider
        {
            get
            {
                if (!string.IsNullOrEmpty(ProfileGUID))
                    return PluginHandler.GetPluginProvider(ProfileGUID);
                else
                    return null;
            }
        }

        #endregion
    }// class
}// namespace