﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.Data;
using System.IO;
using SmarterNotify.HelperClasses;
using System.Xml;

namespace SmarterNotify.DataHandlers
{
    public class ConnectionProfileHandler
    {
        public static Dictionary<string, ConnectionProfile> Profiles = new Dictionary<string, ConnectionProfile>();
        public static string ProfileLocker = "";

        public static void LoadProfilesFromFile()
        {
            try
            {
                // open a connection to the Connections file and load all the available ones to memory
                LoadProfilesFromFile(AppPath.ProfilesFilePath);
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while loading profiles from file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void CleanProfiles(List<ConnectionProfile> profiles)
        {
            foreach (ConnectionProfile profile in profiles)
            {
                // clean Plugins profiles and their associated internal reminders
                if (profile.ConnectionType != ConnectionProfile.ConnectionProfileType.PluginProvider)
                    continue;

                if (!PluginHandler.IsPluginAvailable(profile.ProfileGUID))
                    DeleteProfile(profile.ProfileGUID);
            }
        }
        public static void LoadProfilesFromFile(string fileLocation)
        {
            lock (ProfileLocker)
            {
                List<ConnectionProfile> profiles = GetProfilesFromFile(fileLocation);
                foreach (ConnectionProfile profile in profiles)
                    Profiles[profile.ProfileGUID] = profile;

                CleanProfiles(profiles);
            }
        }
        public static List<ConnectionProfile> GetProfilesFromFile(string fileLocation)
        {
            List<ConnectionProfile> retVal = new List<ConnectionProfile>();
            FileInfo info = new FileInfo(fileLocation);
            if (info.Extension.ToUpperInvariant().Contains("XML"))
            {
                try
                {
                    if (!File.Exists(fileLocation))
                    {
                        XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                        xmlWrite.WriteStartDocument();
                        xmlWrite.WriteStartElement("Profiles");
                        xmlWrite.WriteEndElement();
                        xmlWrite.Flush();
                        xmlWrite.Close();
                    }

                    XmlTextReader xmlRead = new XmlTextReader(fileLocation);

                    while (xmlRead.Read())
                    {
                        switch (xmlRead.NodeType)
                        {
                            case XmlNodeType.Element:
                                if (xmlRead.Name == "Profile")
                                {
                                    ConnectionProfile profile = InterpretProfile(ref xmlRead);
                                    if (profile != null)
                                        retVal.Add(profile);
                                }
                                break;
                        }
                    }

                    xmlRead.Close();
                }
                catch (Exception ex) 
                {
                    string logMsg = "Error occurred while getting profiles from file";
                    TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
                }
            }
            return retVal;
        }
        public static ConnectionProfile InterpretProfile(ref XmlTextReader xmlRead)
        {
            ConnectionProfile loadedProfile = new ConnectionProfile();
            // initiate to the first node of this profile
            string profileGuid = xmlRead["guid"] == null ? "" : xmlRead["guid"];
            if (string.IsNullOrEmpty(profileGuid))
                return null;
            else
                loadedProfile.ProfileGUID = profileGuid;

            xmlRead.Read();
            while (xmlRead.Name.ToUpperInvariant() != "PROFILE" && xmlRead.NodeType != XmlNodeType.EndElement)
            {
                if (xmlRead.NodeType == XmlNodeType.Element)
                {
                    switch (xmlRead.Name.ToUpperInvariant())
                    {
                        case "PROFILENAME": loadedProfile.ProfileName = xmlRead["value"]; break;
                        case "CONNECTIONTYPE":
                            string conType = xmlRead["value"];
                            if (conType.ToUpperInvariant().Contains("SMARTERMAIL"))
                                loadedProfile.ConnectionType = ConnectionProfile.ConnectionProfileType.SmarterMail;
                            else if (conType.ToUpperInvariant().Contains("SMARTERSTATS"))
                                loadedProfile.ConnectionType = ConnectionProfile.ConnectionProfileType.SmarterStats;
                            else if (conType.ToUpperInvariant().Contains("SMARTERTRACK"))
                                loadedProfile.ConnectionType = ConnectionProfile.ConnectionProfileType.SmarterTrack;
                            else if (conType.ToUpperInvariant().Contains("EXTERNALPROVIDER"))
                                loadedProfile.ConnectionType = ConnectionProfile.ConnectionProfileType.ExternalProvider;
                            else if (conType.ToUpperInvariant().Contains("PLUGINPROVIDER"))
                                loadedProfile.ConnectionType = ConnectionProfile.ConnectionProfileType.PluginProvider;
                            else
                                loadedProfile.ConnectionType = ConnectionProfile.ConnectionProfileType.SmarterTrack;
                            break;
                        case "CONNECTIONURL": loadedProfile.ConnectionURL = xmlRead["value"]; break;
                        case "AUTHUSERNAME": loadedProfile.AuthUsername = xmlRead["value"]; break;
                        case "AUTHPASSWORD": loadedProfile.AuthPassword = Encryption.FromBase64(xmlRead["value"]); break;
                        case "DATAUSERNAME": loadedProfile.DataUsername = xmlRead["value"]; break;
                        case "STARTAPPCONNECTED": loadedProfile.StartAppConnected = xmlRead["value"].ToUpperInvariant().Contains("TRUE") ? true : false; break;
                        case "CUSTOMVARIABLE": 
                            ConnectionProfile.CustomVariableTypes typeKey = ConnectionProfile.CustomVariableTypes.PluginType;
                            switch (xmlRead["type"].ToUpperInvariant())
                            {
                                case "PLUGINTYPE": typeKey = ConnectionProfile.CustomVariableTypes.PluginType; break;
                            }
                            loadedProfile.CustomVars[typeKey] = xmlRead["value"];
                            break;
                    }
                }
                xmlRead.Read();
            }

            if (loadedProfile.ConnectionType == ConnectionProfile.ConnectionProfileType.PluginProvider)
            {
                string plugType = loadedProfile.CustomVars[ConnectionProfile.CustomVariableTypes.PluginType];
                PluginHandler.CreatePluginInstance(loadedProfile.ProfileGUID, plugType == null ? string.Empty : plugType);
            }
            return loadedProfile;
        }
        public static void WriteProfilesToFile()
        {
            // open a connection to the Connections.profile file and load all the available ones to memory
            try
            {
                WriteProfilesToFile(AppPath.ProfilesFilePath, new List<ConnectionProfile>(Profiles.Values));
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occursred while writing profiles to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }

        public static void StartAppConnectedProfiles()
        {
            lock (ProfileLocker)
            {
                foreach (KeyValuePair<string, ConnectionProfile> profile in Profiles)
                {
                    if (profile.Value.StartAppConnected)
                        profile.Value.StartConnections(false);
                }
            }
        }
        public static void WriteProfilesToFile(string fileLocation)
        {
            WriteProfilesToFile(fileLocation, new List<ConnectionProfile>(Profiles.Values));
        }
        public static void WriteProfilesToFile(string fileLocation, List<ConnectionProfile> profiles)
        {
            lock (ProfileLocker)
            {
                XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                xmlWrite.WriteStartDocument();
                xmlWrite.WriteStartElement("Profiles");
                foreach (ConnectionProfile profile in profiles)
                {
                    xmlWrite.WriteStartElement("Profile");
                    xmlWrite.WriteAttributeString("guid", profile.ProfileGUID);

                    xmlWrite.WriteStartElement("ProfileName");
                    xmlWrite.WriteAttributeString("value", profile.ProfileName);
                    xmlWrite.WriteEndElement();

                    xmlWrite.WriteStartElement("ConnectionType");
                    xmlWrite.WriteAttributeString("value", profile.ConnectionType.ToString());
                    xmlWrite.WriteEndElement();

                    xmlWrite.WriteStartElement("ConnectionURL");
                    xmlWrite.WriteAttributeString("value", profile.ConnectionURL);
                    xmlWrite.WriteEndElement();

                    xmlWrite.WriteStartElement("AuthUsername");
                    xmlWrite.WriteAttributeString("value", profile.AuthUsername);
                    xmlWrite.WriteEndElement();

                    xmlWrite.WriteStartElement("AuthPassword");
                    xmlWrite.WriteAttributeString("value", Encryption.ToBase64(profile.AuthPassword));
                    xmlWrite.WriteEndElement();

                    xmlWrite.WriteStartElement("DataUsername");
                    xmlWrite.WriteAttributeString("value", profile.DataUsername);
                    xmlWrite.WriteEndElement();

                    xmlWrite.WriteStartElement("StartAppConnected");
                    xmlWrite.WriteAttributeString("value", profile.StartAppConnected.ToString());
                    xmlWrite.WriteEndElement();

                    foreach (KeyValuePair<ConnectionProfile.CustomVariableTypes, string> item in profile.CustomVars)
                    {
                        xmlWrite.WriteStartElement("CustomVariable");
                        xmlWrite.WriteAttributeString("type", item.Key.ToString());
                        xmlWrite.WriteAttributeString("value", item.Value);
                        xmlWrite.WriteEndElement();
                    }

                    xmlWrite.WriteEndElement();
                }
                xmlWrite.WriteEndElement();
                xmlWrite.Flush();
                xmlWrite.Close();
            }
        }
        public static void ClearProfiles()
        {
            lock (ProfileLocker)
            {
                Profiles.Clear();
            }
        }
        public static ConnectionProfile GetProfile(string profileGUID)
        {
            ConnectionProfile retVal = null;
            lock (ProfileLocker)
            {
                if (Profiles.ContainsKey(profileGUID))
                    retVal = Profiles[profileGUID];
            }
            return retVal;
        }
        public static List<ConnectionProfile> GetAllProfiles()
        {
            List<ConnectionProfile> retVal = new List<ConnectionProfile>();
            lock (ProfileLocker)
            {
                foreach (KeyValuePair<string, ConnectionProfile> profile in Profiles)
                    retVal.Add(profile.Value);
            }
            return retVal;
        }
        private static List<ConnectionProfile> GetConnectionProfiles(ConnectionProfile.ConnectionProfileType profileType)
        {
            List<ConnectionProfile> retVal = new List<ConnectionProfile>();
            lock (ProfileLocker)
            {
                foreach (KeyValuePair<string, ConnectionProfile> profile in Profiles)
                    if (profile.Value.ConnectionType == profileType)
                        retVal.Add(profile.Value);
            }
            return retVal;
        }
        public static List<ConnectionProfile> GetSMProfiles()
        {
            return GetConnectionProfiles(ConnectionProfile.ConnectionProfileType.SmarterMail);
        }
        public static List<ConnectionProfile> GetSTProfiles()
        {
            return GetConnectionProfiles(ConnectionProfile.ConnectionProfileType.SmarterTrack);
        }
        public static List<ConnectionProfile> GetSSProfiles()
        {
            return GetConnectionProfiles(ConnectionProfile.ConnectionProfileType.SmarterStats);
        }
        public static List<ConnectionProfile> GetExternalProfiles()
        {
            return GetConnectionProfiles(ConnectionProfile.ConnectionProfileType.ExternalProvider);
        }
        public static List<ConnectionProfile> GetPluginProfiles()
        {
            return GetConnectionProfiles(ConnectionProfile.ConnectionProfileType.PluginProvider);
        }

        public static void SaveProfile(ConnectionProfile profile, bool writeToFile)
        {
            SaveProfile(profile, writeToFile, true);
        }
        public static void SaveProfile(ConnectionProfile profile, bool writeToFile, bool writeToLog)
        {
            lock (ProfileLocker)
            {
                ConnectionProfile conProf = new ConnectionProfile();
                if (writeToLog)
                {
                    if (Profiles.TryGetValue(profile.ProfileGUID, out conProf))
                    {
                        string logMsg = "Connection profile \"" + profile.ProfileName + "\" has been saved";
                        TrackingHelper.TrackConnection(TrackingHelper.RecordType.Log, "Connection Profile Saved", logMsg, logMsg);
                    }
                    else
                    {
                        string logMsg = "Connection profile \"" + profile.ProfileName + "\" has been created";
                        TrackingHelper.TrackConnection(TrackingHelper.RecordType.Log, "Connection Profile Created", logMsg, logMsg);
                    }
                }
                Profiles[profile.ProfileGUID] = profile;
                if (writeToFile)
                    WriteProfilesToFile();
            }
        }
        public static void DeleteProfile(string profileGUID)
        {
            lock (ProfileLocker)
            {
                if (Profiles.ContainsKey(profileGUID))
                {
                    ConnectionProfile profile = Profiles[profileGUID];
                    string logMsg = "The connection profile \"" + profile.ProfileName + "\" has been deleted";
                    if (profile.ConnectionType == ConnectionProfile.ConnectionProfileType.PluginProvider)
                    {
                        profile.DismissAllReminders();
                        profile.StopConnections();
                        PluginHandler.RemoveProviderInstance(profileGUID);
                        InternalReminderHandler.WriteRemindersToFile();
                    }
                    TrackingHelper.TrackConnection(TrackingHelper.RecordType.Log, "Connection Profile Deleted", logMsg, logMsg);
                    Profiles.Remove(profileGUID);
                    WriteProfilesToFile();
                }
            }
        }
    }
}