﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.Data;
using SmarterNotify.HelperClasses;
using System.IO;
using System.Xml;

namespace SmarterNotify.DataHandlers
{
    public class InternalReminderHandler
    {
        private static Dictionary<string, Reminder> Reminders = new Dictionary<string, Reminder>();
        private static object RemindersLocker = new object();

        public static void LoadRemindersFromFile()
        {
            try
            {
                // open a connection to the internal reminders file and load all the available ones to memory
                LoadRemindersFromFile(AppPath.InternalAlertsFilePath);
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while loading internal reminders from file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void LoadRemindersFromFile(string fileLocation)
        {
            lock (RemindersLocker)
            {
                foreach (Reminder reminder in GetRemindersFromFile(fileLocation))
                    Reminders[reminder.ReminderID] = reminder;
            }
        }
        public static List<Reminder> GetRemindersFromFile(string fileLocation)
        {
            List<Reminder> retVal = new List<Reminder>();
            FileInfo info = new FileInfo(fileLocation);
            if (info.Extension.ToUpperInvariant().Contains("XML"))
            {
                try
                {
                    if (!File.Exists(fileLocation))
                    {
                        XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                        xmlWrite.WriteStartDocument();
                        xmlWrite.WriteStartElement("Reminders");
                        xmlWrite.WriteEndElement();
                        xmlWrite.Flush();
                        xmlWrite.Close();
                    }

                    XmlTextReader xmlRead = new XmlTextReader(fileLocation);

                    while (xmlRead.Read())
                    {
                        switch (xmlRead.NodeType)
                        {
                            case XmlNodeType.Element:
                                if (xmlRead.Name == "Reminder")
                                {
                                    Reminder reminder = InterpretReminder(ref xmlRead);
                                    if (reminder != null)
                                        retVal.Add(reminder);
                                }
                                break;
                        }
                    }

                    xmlRead.Close();
                }
                catch (Exception ex) 
                {
                    string logMsg = "Error occurred while getting internal reminders from file";
                    TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
                }
            }
            return retVal;
        }
        public static Reminder InterpretReminder(ref XmlTextReader xmlRead)
        {
            Reminder loadedReminder = new Reminder();
            // initiate to the first node of this profile
            string reminderGUID = xmlRead["guid"] == null ? "" : xmlRead["guid"];
            if (string.IsNullOrEmpty(reminderGUID))
                return null;
            else
                loadedReminder.ReminderID = reminderGUID;

            DateTime dtParse = DateTime.MinValue;
            xmlRead.Read();
            while (xmlRead.Name.ToUpperInvariant() != "REMINDER" && xmlRead.NodeType != XmlNodeType.EndElement)
            {
                if (xmlRead.NodeType == XmlNodeType.Element)
                {
                    switch (xmlRead.Name.ToUpperInvariant())
                    {
                        case "SUBJECT": loadedReminder.Subject = xmlRead["value"]; break;
                        case "PHONICSUBJECT": loadedReminder.PhonicSubject = xmlRead["value"]; break;
                        case "BODYTEXT": loadedReminder.BodyText = xmlRead["value"]; break;
                        case "SNOOZETIMEUTC": 
                            if (DateTime.TryParse(xmlRead["value"], out dtParse) && dtParse > DateTime.MinValue)
                                loadedReminder.SnoozeTimeUTC = dtParse; 
                            break;
                        case "TIMEOCCURREDUTC":
                            if (DateTime.TryParse(xmlRead["value"], out dtParse) && dtParse > DateTime.MinValue)
                                loadedReminder.TimeOccurredUTC = dtParse;
                            break;
                        case "OWNERPROFILENAME": loadedReminder.OwnerProfileName = xmlRead["value"]; break;
                        case "REMINDERTYPE": 
                            switch (xmlRead["value"].ToUpperInvariant())
                            {
                                case "INTERNAL": loadedReminder.ReminderType = Reminder.AlertType.Internal; break;
                                case "WEBSERVICE": loadedReminder.ReminderType = Reminder.AlertType.WebService; break;
                                case "PLUGIN": loadedReminder.ReminderType = Reminder.AlertType.Plugin; break;
                            }
                            break;
                        case "PROFILEGUID": loadedReminder.ProfileGUID = xmlRead["value"]; break;
                    }
                }
                xmlRead.Read();
            }
            return loadedReminder;
        }
        public static void WriteRemindersToFile()
        {
            // open a connection to the Connections.profile file and load all the available ones to memory
            try
            {
                WriteRemindersToFile(AppPath.InternalAlertsFilePath, new List<Reminder>(Reminders.Values));
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while writing internal reminders to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void WriteRemindersToFile(string fileLocation)
        {
            try
            {
                WriteRemindersToFile(fileLocation, new List<Reminder>(Reminders.Values));
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while writing internal reminders to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void WriteRemindersToFile(string fileLocation, List<Reminder> reminders)
        {
            try
            {
                lock (RemindersLocker)
                {
                    XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                    xmlWrite.WriteStartDocument();
                    xmlWrite.WriteStartElement("Reminders");
                    foreach (Reminder reminder in reminders)
                    {
                        xmlWrite.WriteStartElement("Reminder");
                        xmlWrite.WriteAttributeString("guid", reminder.ReminderID);

                        xmlWrite.WriteStartElement("Subject");
                        xmlWrite.WriteAttributeString("value", reminder.Subject);
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("PhonicSubject");
                        xmlWrite.WriteAttributeString("value", reminder.PhonicSubject);
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("BodyText");
                        xmlWrite.WriteAttributeString("value", reminder.BodyText);
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("SnoozeTimeUTC");
                        xmlWrite.WriteAttributeString("value", reminder.SnoozeTimeUTC.ToString());
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("TimeOccurredUTC");
                        xmlWrite.WriteAttributeString("value", reminder.TimeOccurredUTC.ToString());
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("OwnerProfileName");
                        xmlWrite.WriteAttributeString("value", reminder.OwnerProfileName);
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("ReminderType");
                        xmlWrite.WriteAttributeString("value", reminder.ReminderType.ToString());
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("ProfileGUID");
                        xmlWrite.WriteAttributeString("value", reminder.ProfileGUID.ToString());
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteEndElement();
                    }
                    xmlWrite.WriteEndElement();
                    xmlWrite.Flush();
                    xmlWrite.Close();
                }
            }
            catch (Exception ex) 
            {
                string logMsg = "Error occurred while writing internal reminders to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static Reminder GetInternalReminder(string id)
        {
            Reminder reminder = null;
            if (SettingsHandler.GlobalSettings.ShouldShowSmarterNotify())
            {
                lock (RemindersLocker)
                {
                    Reminders.TryGetValue(id, out reminder);
                }
            }
            return reminder;
        }
        public static Reminder GetPluginReminder(string id)
        {
            Reminder reminder = null;
            lock (RemindersLocker)
            {
                Reminders.TryGetValue(id, out reminder);
                ConnectionProfile profile = ConnectionProfileHandler.GetProfile(reminder.ProfileGUID);
                bool shouldReturnNull = true;
                if (profile != null)
                {
                    if (profile.ConnectionType == ConnectionProfile.ConnectionProfileType.PluginProvider)
                        shouldReturnNull = false;
                }
                if (shouldReturnNull)
                    reminder = null;
            }
            return reminder;
        }
        public static List<Reminder> GetAllPluginReminders(bool showSnoozedItems)
        {
            List<Reminder> reminders = new List<Reminder>();
            lock (RemindersLocker)
            {
                foreach (KeyValuePair<string, Reminder> item in Reminders)
                {
                    if (item.Value.ReminderType == Reminder.AlertType.Plugin)
                    {
                        bool isSnoozing = item.Value.SnoozeTimeUTC > DateTime.UtcNow;
                        if (!showSnoozedItems && isSnoozing)
                            continue;
                        else
                            reminders.Add(item.Value);

                        if (!isSnoozing)
                            SnoozeReminderMethod(item.Value.ReminderID, -1);
                    }
                }
            }
            return reminders;
        }
        public static List<Reminder> GetAllInternalReminders(bool showSnoozedItems)
        {
            List<Reminder> reminders = new List<Reminder>();
            lock (RemindersLocker)
            {
                foreach (KeyValuePair<string, Reminder> item in Reminders)
                {
                    if (item.Value.ReminderType == Reminder.AlertType.Internal)
                    {
                        bool isSnoozing = item.Value.SnoozeTimeUTC > DateTime.UtcNow;
                        if (!showSnoozedItems && isSnoozing)
                            continue;
                        else
                            reminders.Add(item.Value);

                        if (!isSnoozing)
                            SnoozeReminderMethod(item.Value.ReminderID, -1);
                    }
                }
            }
            return reminders;
        }
        public static List<Reminder> GetAllReminders(bool showSnoozedItems)
        {
            List<Reminder> reminders = new List<Reminder>();
            if (SettingsHandler.GlobalSettings.ShouldShowSmarterNotify())
            {
                lock (RemindersLocker)
                {
                    foreach (KeyValuePair<string, Reminder> item in Reminders)
                    {
                        bool isSnoozing = item.Value.SnoozeTimeUTC > DateTime.UtcNow;
                        if (!showSnoozedItems && isSnoozing)
                            continue;
                        else
                            reminders.Add(item.Value);

                        if (!isSnoozing)
                            SnoozeReminderMethod(item.Value.ReminderID, -1);
                    }
                }
            }
            return reminders;
        }
        public static void SaveReminder(Reminder reminder, bool writeToFile)
        {
            lock (RemindersLocker)
            {
                Reminders[reminder.ReminderID] = reminder;
                if (writeToFile)
                    WriteRemindersToFile();
            }
        }
        public static void DismissReminder(string id, bool writeToFile)
        {
            lock (RemindersLocker)
            {
                Reminder reminder = null;
                if (Reminders.TryGetValue(id, out reminder) && reminder != null)
                    Reminders.Remove(id);
                if (writeToFile)
                    WriteRemindersToFile();
            }
        }
        public static void DismissAllInternalReminders(bool writeToFile)
        {
            lock (RemindersLocker)
            {
                DismissAllOnType(Reminder.AlertType.Internal);

                if (writeToFile)
                    WriteRemindersToFile();
            }
        }
        public static void DismissAllPluginReminders(bool writeToFile)
        {
            lock (RemindersLocker)
            {
                DismissAllOnType(Reminder.AlertType.Plugin);

                if (writeToFile)
                    WriteRemindersToFile();
            }
        }
        private static void DismissAllOnType(Reminder.AlertType disType)
        {
            List<string> keysToRemove = new List<string>();
            foreach (KeyValuePair<string, Reminder> item in Reminders)
                if (item.Value.ReminderType == disType)
                    keysToRemove.Add(item.Key);

            foreach (string key in keysToRemove)
                Reminders.Remove(key);
        }

        public static void SnoozeReminder(string id, int snoozeSeconds, bool writeToFile)
        {
            lock (RemindersLocker)
            {
                SnoozeReminderMethod(id, snoozeSeconds);
                if (writeToFile)
                    WriteRemindersToFile();
            }
        }
        private static void SnoozeReminderMethod(string id, int snoozeSeconds)
        {
            Reminder reminder = null;
            if (Reminders.TryGetValue(id, out reminder) && reminder != null)
            {
                if (snoozeSeconds <= 0)
                    reminder.SnoozeTimeUTC = DateTime.MinValue;
                else
                    reminder.SnoozeTimeUTC = DateTime.UtcNow.AddSeconds((double)snoozeSeconds);
            }
        }
    }
}