﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.Connector;
using System.Reflection;
using System.Windows.Forms;
using SmarterNotify.HelperClasses;
using System.IO;
using SmarterNotify.Data;

namespace SmarterNotify.DataHandlers
{
    public class PluginHandler
    {
        // type, file (filled out on load)
        private static Dictionary<string, string> PluginTypeNames = new Dictionary<string, string>();
        // file, assembly reference (filled out on load)
        private static Dictionary<string, Assembly> PluginProviderAssemblies = new Dictionary<string, Assembly>();
        // profile GUID, plugin provider
        private static Dictionary<string, INotifyPluginProvider> PluginProviderInstances = new Dictionary<string, INotifyPluginProvider>();
        private static object PluginLocker = new object();

        public static void LoadPluginProviders()
        {
            List<string> types = new List<string>();

            lock (PluginLocker)
            {
                List<string> filesToSearch = new List<string>();
                foreach (string fyl in Directory.GetFiles(AppPath.PluginPath))
                    filesToSearch.Add(fyl);

                foreach (string fyl in filesToSearch)
                {
                    FileInfo fylInfo = new FileInfo(fyl);
                    if (fylInfo.Extension.ToUpperInvariant() == ".DLL" || fylInfo.Extension.ToUpperInvariant() == ".EXE")
                    {
                        Assembly u = Assembly.LoadFile(fyl);
                        foreach (var t in u.GetTypes())
                        {
                            if (t.GetInterface("SmarterNotify.Connector.INotifyPluginProvider", true) != null)
                                types.Add(t.ToString());

                            Type typ = u.GetType(t.ToString());
                            if (typ != null)
                            {
                                PluginTypeNames[typ.ToString()] = fyl;
                            }
                        }
                        PluginProviderAssemblies[fyl] = u;
                    }
                }
            }
        }
        public static void CreatePluginInstance(string profileGUID, string type)
        {
            if (string.IsNullOrEmpty(type))
                return;

            string fileName = string.Empty;
            PluginTypeNames.TryGetValue(type, out fileName);
            if (fileName != null)
            {
                // get the assembly based on filename
                Assembly u;
                PluginProviderAssemblies.TryGetValue(fileName, out u);
                if (u != null)
                {
                    INotifyPluginProvider plgProvider = u.CreateInstance(type.ToString()) as INotifyPluginProvider;
                    plgProvider.AddNotification += new NotifyHelper.AddNotificationDel(plgProvider_AddNotification);
                    PluginProviderInstances[profileGUID] = plgProvider;
                }
            }
        }

        public static void RemoveProviderInstance(string profileGUID)
        {
            lock (PluginLocker)
            {
                if (PluginProviderInstances.ContainsKey(profileGUID))
                    PluginProviderInstances.Remove(profileGUID);
            }
        }

        public static Dictionary<string, INotifyPluginProvider> GetPluginProviderInstances()
        {
            lock (PluginLocker)
            {
                return PluginProviderInstances;
            }
        }

        public static List<string> GetTypesList()
        {
            List<string> retVal = new List<string>();
            lock (PluginLocker)
            {
                foreach (KeyValuePair<string, string> pair in PluginTypeNames)
                {
                    FileInfo inf = new FileInfo(pair.Value);
                    retVal.Add(inf.Name + " (" + pair.Key + ")");
                }
            }
            return retVal;
        }

        public static void plgProvider_AddNotification(object sender, NotificationEventArgs e)
        {
            INotifyPluginProvider plgProvider = (INotifyPluginProvider)sender;
            string profileGUID = string.Empty;
            if (plgProvider != null)
            {
                foreach (KeyValuePair<string, INotifyPluginProvider> item in PluginProviderInstances)
                {
                    if (plgProvider.Equals(item.Value))
                    {
                        profileGUID = item.Key;
                        break;
                    }
                }
            }
            if (profileGUID.Length > 0)
            {
                Reminder evtReminder = new Reminder();
                foreach (ReminderValuePair pair in e.Notification.ReminderAttributes)
                {
                    DateTime dtParse = DateTime.MinValue;
                    switch (pair.Key)
                    {
                        case ReminderValuePair.ReminderAttributes.Subject:
                            evtReminder.Subject = pair.Val;
                            break;
                        case ReminderValuePair.ReminderAttributes.PhonicSubject:
                            evtReminder.PhonicSubject = pair.Val;
                            break;
                        case ReminderValuePair.ReminderAttributes.CategoryType:
                            evtReminder.CategoryType = pair.Val;
                            break;
                        case ReminderValuePair.ReminderAttributes.BodyText:
                            evtReminder.BodyText = pair.Val;
                            break;
                        case ReminderValuePair.ReminderAttributes.TimeOccurredUTC:
                            DateTime.TryParse(pair.Val, out dtParse);
                            evtReminder.TimeOccurredUTC = dtParse;
                            break;
                        case ReminderValuePair.ReminderAttributes.SnoozeTimeUTC:
                            DateTime.TryParse(pair.Val, out dtParse);
                            evtReminder.SnoozeTimeUTC = dtParse;
                            break;
                    }
                }

                ConnectionProfile profile = ConnectionProfileHandler.GetProfile(profileGUID);
                if (profile != null)
                    evtReminder.OwnerProfileName = profile.ProfileName;
                else
                    evtReminder.OwnerProfileName = "Plugin Providers";

                evtReminder.ProfileGUID = profileGUID;
                evtReminder.ReminderType = Reminder.AlertType.Plugin;
                InternalReminderHandler.SaveReminder(evtReminder, true);
            }
        }
        public static List<INotifyPluginProvider> GetAllPluginProviders()
        {
            List<INotifyPluginProvider> retVal = new List<INotifyPluginProvider>();
            lock (PluginLocker)
            {
                foreach (KeyValuePair<string, INotifyPluginProvider> plugin in PluginProviderInstances)
                    retVal.Add(plugin.Value);
            }
            return retVal;
        }
        public static INotifyPluginProvider GetPluginProvider(string id)
        {
            INotifyPluginProvider retVal;
            lock (PluginLocker)
            {
                PluginProviderInstances.TryGetValue(id, out retVal);
            }
            return retVal;
        }
        public static bool IsPluginAvailable(string id)
        {
            lock (PluginLocker)
            {
                if (id == null)
                    return false;

                INotifyPluginProvider provider;
                if (PluginProviderInstances.TryGetValue(id, out provider) && provider != null)
                    return true;

                return false;
            }
        }
    }// class
}// namespace