﻿using System;
using System.Collections.Generic;
using System.Text;
using SmarterNotify.Data;
using SmarterNotify.HelperClasses;
using System.IO;
using System.Xml;
using SmarterNotify.Data.Providers;

namespace SmarterNotify.DataHandlers
{
    public class ProviderHandler
    {
        public static Dictionary<string, Provider> Providers = new Dictionary<string, Provider>();
        public static object ProviderLocker = new object();

        public static void LoadProvidersFromFile()
        {
            try
            {
                // open a connection to the providers file and load all the available ones to memory
                LoadProvidersFromFile(AppPath.ProviderFilePath);
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while loading providers from file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void LoadProvidersFromFile(string fileLocation)
        {
            lock (ProviderLocker)
            {
                foreach (Provider provider in GetProvidersFromFile(fileLocation))
                    Providers[provider.ProviderGUID] = provider;
            }
        }
        public static List<Provider> GetProvidersFromFile(string fileLocation)
        {
            List<Provider> retVal = new List<Provider>();
            FileInfo info = new FileInfo(fileLocation);
            if (info.Extension.ToUpperInvariant().Contains("XML"))
            {
                try
                {
                    if (!File.Exists(fileLocation))
                    {
                        XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                        xmlWrite.WriteStartDocument();
                        xmlWrite.WriteStartElement("Providers");
                        xmlWrite.WriteEndElement();
                        xmlWrite.Flush();
                        xmlWrite.Close();
                    }

                    XmlTextReader xmlRead = new XmlTextReader(fileLocation);

                    while (xmlRead.Read())
                    {
                        switch (xmlRead.NodeType)
                        {
                            case XmlNodeType.Element:
                                if (xmlRead.Name == "Provider")
                                {
                                    Provider provider = InterpretProvider(ref xmlRead);
                                    if (provider != null)
                                        retVal.Add(provider);
                                }
                                break;
                        }
                    }

                    xmlRead.Close();
                }
                catch (Exception ex)
                {
                    string logMsg = "Error occurred while getting providers from file";
                    TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
                }
            }
            return retVal;
        }
        public static Provider InterpretProvider(ref XmlTextReader xmlRead)
        {
            Provider loadedProvider = new Provider();
            // initiate to the first node of this profile
            string providerGUID = xmlRead["guid"] == null ? "" : xmlRead["guid"];
            if (string.IsNullOrEmpty(providerGUID))
                return null;
            else
                loadedProvider.ProviderGUID = providerGUID;

            bool boolParse = false;
            xmlRead.Read();
            while (xmlRead.Name.ToUpperInvariant() != "PROVIDER" && xmlRead.NodeType != XmlNodeType.EndElement)
            {
                if (xmlRead.NodeType == XmlNodeType.Element)
                {
                    switch (xmlRead.Name.ToUpperInvariant())
                    {
                        case "PROVIDERNAME": loadedProvider.ProviderName = xmlRead["value"]; break;
                        case "PROVIDERURL": loadedProvider.ProviderURL = xmlRead["value"]; break;
                        case "ENABLED":
                            if (bool.TryParse(xmlRead["value"], out boolParse))
                                loadedProvider.Enabled = boolParse;
                            break;
                        case "PROVIDERFIELDS":
                            while (xmlRead.Name.ToUpperInvariant() != "PROVIDERFIELDS" && xmlRead.NodeType != XmlNodeType.EndElement)
                            {
                                ReadProviderField(ref xmlRead, ref loadedProvider);
                                if (xmlRead.EOF)
                                    break;
                            }
                            break;
                    }
                }
                xmlRead.Read();
            }

            return loadedProvider;
        }
        public static void ReadProviderField(ref XmlTextReader xmlRead, ref Provider rProvider)
        {
            xmlRead.Read();
            ProviderField providerField = new ProviderField();
            while (xmlRead.Name.ToUpperInvariant() != "PROVIDERFIELD" && xmlRead.NodeType != XmlNodeType.EndElement)
            {
                if (xmlRead.NodeType == XmlNodeType.Element)
                {
                    switch (xmlRead.Name.ToUpperInvariant())
                    {
                        case "KEY": providerField.Key = xmlRead["value"]; break;
                        case "VAL": providerField.Val = xmlRead["value"]; break;
                    }
                }
                xmlRead.Read();
            }
            rProvider.ProviderFields.Add(providerField);
        }
        public static void WriteProvidersToFile()
        {
            // open a connection to the Connections.profile file and load all the available ones to memory
            try
            {
                WriteProvidersToFile(AppPath.ProviderFilePath, new List<Provider>(Providers.Values));
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while writing providers to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void WriteProvidersToFile(string fileLocation)
        {
            try
            {
                WriteProvidersToFile(fileLocation, new List<Provider>(Providers.Values));
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while writing providers to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }
        public static void WriteProvidersToFile(string fileLocation, List<Provider> providers)
        {
            try
            {
                lock (ProviderLocker)
                {
                    XmlTextWriter xmlWrite = new XmlTextWriter(fileLocation, null);
                    xmlWrite.WriteStartDocument();
                    xmlWrite.WriteStartElement("Providers");
                    foreach (Provider provider in providers)
                    {
                        xmlWrite.WriteStartElement("Provider");
                        xmlWrite.WriteAttributeString("guid", provider.ProviderGUID);

                        xmlWrite.WriteStartElement("ProviderName");
                        xmlWrite.WriteAttributeString("value", provider.ProviderName);
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("ProviderURL");
                        xmlWrite.WriteAttributeString("value", provider.ProviderURL);
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("Enabled");
                        xmlWrite.WriteAttributeString("value", provider.Enabled.ToString());
                        xmlWrite.WriteEndElement();

                        xmlWrite.WriteStartElement("ProviderFields");
                        foreach (ProviderField field in provider.ProviderFields)
                        {
                            xmlWrite.WriteStartElement("ProviderField");

                            xmlWrite.WriteStartElement("Key");
                            xmlWrite.WriteAttributeString("value", field.Key);
                            xmlWrite.WriteEndElement();

                            xmlWrite.WriteStartElement("Val");
                            xmlWrite.WriteAttributeString("value", field.Val);
                            xmlWrite.WriteEndElement();

                            xmlWrite.WriteEndElement();
                        }
                        xmlWrite.WriteEndElement();
                        xmlWrite.WriteEndElement();
                    }
                    xmlWrite.WriteEndElement();
                    xmlWrite.Flush();
                    xmlWrite.Close();
                }
            }
            catch (Exception ex)
            {
                string logMsg = "Error occurred while writing providers to file";
                TrackingHelper.TrackError(TrackingHelper.RecordType.Both, "Error Occurred", logMsg, logMsg, ex);
            }
        }

        public static Provider GetProvider(string id)
        {
            Provider provider = null;
            lock (ProviderLocker)
            {
                Providers.TryGetValue(id, out provider);
            }
            return provider;
        }
        public static void SaveProvider(Provider provider, bool writeToFile)
        {
            lock (ProviderLocker)
            {
                Providers[provider.ProviderGUID] = provider;
                if (writeToFile)
                    WriteProvidersToFile();
            }
        }
        public static void DeleteProvider(string id)
        {
            lock (ProviderLocker)
            {
                Provider provider = null;
                if (Providers.TryGetValue(id, out provider) && provider != null)
                    Providers.Remove(id);
            }
        }
    }
}