﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SmarterNotify.Data;
using SmarterNotify.DataHandlers;
using SmarterNotify.HelperClasses;
using SmarterNotify.Connector;
using SmarterNotify.Threads;

namespace SmarterNotify
{
    public partial class frmConnectionProfile : Form
    {
        private string profileGUID;
        public delegate void UpdateParentList();
        public event UpdateParentList OnUpdate;

        public bool ShouldAddProfile = false;

        public frmConnectionProfile(string profileGUID)
        {
            this.profileGUID = profileGUID;
            InitializeComponent();
        }

        private void frmConnectionProfile_Load(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(profileGUID))
            {
                // pre load the form if this is in edit mode
                ConnectionProfile profile = ConnectionProfileHandler.GetProfile(profileGUID);
                txtProfileName.Text = profile.ProfileName;
                chkStartConnected.Checked = profile.StartAppConnected;
                switch (profile.ConnectionType)
                {
                    case ConnectionProfile.ConnectionProfileType.SmarterMail:
                        cboConnectionType.SelectedItem = "SmarterMail 5.5 and later";
                        break;
                    case ConnectionProfile.ConnectionProfileType.SmarterTrack:
                        cboConnectionType.SelectedItem = "SmarterTrack 3.6 and later";
                        break;
                    case ConnectionProfile.ConnectionProfileType.SmarterStats:
                        cboConnectionType.SelectedItem = "SmarterStats";
                        break;
                    case ConnectionProfile.ConnectionProfileType.ExternalProvider:
                        cboConnectionType.SelectedItem = "External Provider";
                        break;
                    case ConnectionProfile.ConnectionProfileType.PluginProvider:
                        cboConnectionType.SelectedItem = "Plugin Provider";
                        break;
                }

                txtAgentUsername.Text = profile.DataUsername;
                txtAuthUsername.Text = profile.AuthUsername;
                txtAuthPassword.Text = profile.AuthPassword;
                txtConnectionURL.Text = profile.ConnectionURL;

                foreach (KeyValuePair<ConnectionProfile.CustomVariableTypes, string> item in profile.CustomVars)
                {
                    switch (item.Key)
                    {
                        case ConnectionProfile.CustomVariableTypes.PluginType:
                            cboPluginType.SelectedItem = item.Value;
                            break;
                    }
                }
            }
            else
            {
                if (cboConnectionType.Items.Count > 0)
                    cboConnectionType.SelectedIndex = 0;
            }

            ReflectPlugins();
            cboConnectionType_SelectedIndexChanged(null, null);
        }
        private void btnHelp_Click(object sender, EventArgs e)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.ConnectionProfile);
        }
        private void frmConnectionProfile_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            ViewManager.OpenHelp(ViewManager.HelpCriteria.ConnectionProfile);
        }
        private void btnSave_Click(object sender, EventArgs e)
        {
            ConnectionProfile profile = new ConnectionProfile();
            bool editingProfile = !string.IsNullOrEmpty(profileGUID);
            if (editingProfile)
                profile = ConnectionProfileHandler.GetProfile(profileGUID);

            if (txtProfileName.Text.Length == 0)
            {
                MessageBox.Show("The profile name cannot be blank",
                    "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            profile.ProfileName = txtProfileName.Text;
            profile.StartAppConnected = chkStartConnected.Checked;

            switch (cboConnectionType.SelectedItem.ToString().ToUpperInvariant())
            {
                case "SMARTERMAIL 5.5 AND LATER": 
                    profile.ConnectionType = ConnectionProfile.ConnectionProfileType.SmarterMail;
                    profile.ConnectionURL = MakeConnectionURL(txtConnectionURL.Text);
                    break;
                case "SMARTERTRACK 3.6 AND LATER": 
                    profile.ConnectionType = ConnectionProfile.ConnectionProfileType.SmarterTrack;
                    profile.ConnectionURL = MakeConnectionURL(txtConnectionURL.Text);
                    break;
                case "SMARTERSTATS": 
                    profile.ConnectionType = ConnectionProfile.ConnectionProfileType.SmarterStats;
                    profile.ConnectionURL = MakeConnectionURL(txtConnectionURL.Text);
                    break;
                case "WEB SERVICE PROVIDER":
                    profile.ConnectionType = ConnectionProfile.ConnectionProfileType.ExternalProvider;
                    profile.ConnectionURL = txtConnectionURL.Text;
                    break;
                case "PLUGIN PROVIDER":
                    profile.ConnectionType = ConnectionProfile.ConnectionProfileType.PluginProvider;
                    if (cboPluginType.Items.Count == 0 || !cboPluginType.Visible)
                    {
                        MessageBox.Show("A plugin provider must be chosen. If none are available, you must add them first to your Plugins directory and restart the application.",
                            "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    else
                    {
                        string pluginString = cboPluginType.SelectedItem.ToString();
                        string namespaceType = pluginString.Substring(pluginString.IndexOf("(") + 1, (pluginString.LastIndexOf(")") - (pluginString.IndexOf("(") + 1)));
                        if (!string.IsNullOrEmpty(namespaceType))
                            profile.CustomVars[ConnectionProfile.CustomVariableTypes.PluginType] = namespaceType;
                        else
                            profile.CustomVars[ConnectionProfile.CustomVariableTypes.PluginType] = cboPluginType.SelectedItem.ToString();

                        PluginHandler.CreatePluginInstance(profile.ProfileGUID, profile.CustomVars[ConnectionProfile.CustomVariableTypes.PluginType]);
                    }
                    break;
            }

            profile.DataUsername = txtAgentUsername.Text;
            profile.AuthUsername = txtAuthUsername.Text;
            profile.AuthPassword = txtAuthPassword.Text;

            if (!profile.IsServiceAvailable())
            {
                frmConnectionProfileConflict conflict = new frmConnectionProfileConflict();
                conflict.OnAction += new frmConnectionProfileConflict.OnActionDel(conflict_OnAction);
                conflict.ConflictedProfile = profile;
                conflict.TopMost = false;
                conflict.ShowDialog();
            }
            else
                ShouldAddProfile = true;

            if (ShouldAddProfile)
            {
                if (!editingProfile)
                    profile.StartConnections(false);

                UpdateThread.CurrentProfileName = profile.ProfileGUID;
                ConnectionProfileHandler.SaveProfile(profile, true);
                OnUpdate();
                this.Close();
            }
        }

        public string MakeConnectionURL(string serverAddr)
        {
            string retVal = serverAddr;

            if (!retVal.StartsWith("http://") && !retVal.StartsWith("https://"))
                retVal = "http://" + retVal;

            if (retVal.EndsWith("Services/svcNotifications.asmx"))
                return retVal;

             if (!retVal.EndsWith("/"))
                retVal += "/";

            retVal += "Services/svcNotifications.asmx";
            return retVal;
        }

        public void conflict_OnAction(bool toggle)
        {
            ShouldAddProfile = toggle;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void cboConnectionType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboConnectionType.SelectedItem != null)
            {
                switch (cboConnectionType.SelectedItem.ToString().ToUpperInvariant())
                {
                    case "SMARTERMAIL 5.5 AND LATER":
                        ShowSmarterMailViewForm();
                        break;
                    case "SMARTERTRACK 3.6 AND LATER":
                        ShowSmarterTrackViewForm();
                        break;
                    case "SMARTERSTATS":
                        break;
                    case "WEB SERVICE PROVIDER":
                        ShowExternalProviderForm();
                        break;
                    case "PLUGIN PROVIDER":
                        ShowPluginProviderForm();
                        break;
                }
            }
        }

        #region Helper Methods

        public void ShowPluginProviderForm()
        {
            chkStartConnected.Visible = true;
            chkStartConnected.Location = new Point(156, 94);

            txtAgentUsername.Visible = false;
            txtAuthPassword.Visible = false;
            txtAuthUsername.Visible = false;
            txtConnectionURL.Visible = false;
            lblDataName.Visible = false;
            lblConnectionExample.Visible = false;
            lblConnectionURL.Visible = false;
            lblAuthorizedPassword.Visible = false;
            lblAuthorizedUsername.Visible = false;
            cboPluginType.Visible = true;
            lblPluginType.Visible = true;

            ReflectPlugins();
        }
        public void ShowSmarterMailViewForm()
        {
            ShowProductComponents();
            lblDataName.Text = "Email Address";
        }
        public void ShowSmarterTrackViewForm()
        {
            ShowProductComponents();
            lblDataName.Text = "Agent Username";
        }
        public void ShowExternalProviderForm()
        {
            lblConnectionURL.Text = "Connection URL";
            lblDataName.Text = "Data Username";
            lblConnectionExample.Text = "(Example: http://example.com/svcNotify.asmx)";
            chkStartConnected.Visible = true;
            chkStartConnected.Location = new Point(156, 193);

            txtAgentUsername.Visible = true;
            txtAuthPassword.Visible = true;
            txtAuthUsername.Visible = true;
            txtConnectionURL.Visible = true;
            lblAuthorizedUsername.Visible = true;
            lblAuthorizedPassword.Visible = true;
            lblDataName.Visible = true;
            lblConnectionExample.Visible = true;
            lblConnectionURL.Visible = true;
            lblAuthorizedPassword.Visible = true;
            lblAuthorizedUsername.Visible = true;
            cboPluginType.Visible = false;
            lblPluginType.Visible = false;
            lblNoneAvailable.Visible = false;
        }
        
        private void ReflectPlugins()
        {
            cboPluginType.Items.Clear();
            foreach (string provider in PluginHandler.GetTypesList())
                cboPluginType.Items.Add(provider);
            if (cboPluginType.Items.Count > 0)
            {
                cboPluginType.SelectedIndex = 0;
                lblNoneAvailable.Visible = false;
                cboPluginType.Visible = true;
            }
            else
            {
                lblNoneAvailable.Visible = true;
                cboPluginType.Visible = false;
            }
        }
        private void ShowProductComponents()
        {
            lblConnectionURL.Text = "Server Address";
            lblConnectionExample.Text = "(Example: http://example.com/)";
            chkStartConnected.Visible = true;
            chkStartConnected.Location = new Point(156, 193);

            txtAgentUsername.Visible = true;
            txtAuthPassword.Visible = true;
            txtAuthUsername.Visible = true;
            txtConnectionURL.Visible = true;
            lblDataName.Visible = true;
            lblConnectionExample.Visible = true;
            lblConnectionURL.Visible = true;
            lblAuthorizedPassword.Visible = true;
            lblAuthorizedUsername.Visible = true;
            cboPluginType.Visible = false;
            lblPluginType.Visible = false;
            lblNoneAvailable.Visible = false;
        }
        #endregion
    }// class
}// namespace